class CalibrationsController < ApplicationController
  include MachineInterfaceable
  include MachineDataViewFilter
  include RxModule::Tableable

  wrap_parameters :calibration, include: [:target, :target_name, :type, :subtype, :mode, :user, :measurements, :corrections, :sn]
  model_data_store(Calibration,
                   {create: false, read: true, update: false, delete: false, index: true},
                   {create: false, read: true, update: false, delete: false, index: true})

  def before_set_targets
    models = if filters["sn"]
               Calibration.where(machine_sn: filters["sn"])
             else
               Calibration
             end
    view_before_set_targets(models, params)
  end

  def before_filter(filter)
    view_before_filter(filter)
  end

  def add
    begin
      MachineInterfaceJobs::CalibrationInterface::AddCalibrationJob.perform_later(add_params)
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  private

  def add_params
    calibration = params.require(:calibration).permit(:target, :target_name, :type, :subtype, :mode, :user)
    correction = params.require(:calibration).permit(corrections: {correction: [:name, :value]}).dig(:corrections)&.dig(:correction) || []
    measurement = params.require(:calibration).permit(measurements: {measurement: [:name, :value]}).dig(:measurements)&.dig(:measurement) || []

    add_params = {
      calibration: {
        target: calibration[:target],
        target_name: calibration[:target_name],
        calibration_type: calibration[:type],
        sub_type: calibration[:subtype],
        mode: calibration[:mode],
        user: calibration[:user],
      },
      measurements: measurement,
      corrections: correction
    }

    add_params.merge! query_params

    add_params
  end
end
