module DataHelpers
  extend ActiveSupport::Concern

  module JobStates
    FINISHED = 'FINISHED'
    STARTED = 'STARTED'
    ABORTED = 'ABORTED'
  end

  MACHINE_STATES = [:on, :notready, :ready, :waitingforjobs, :processing, :maintenance, :error, :halt, :estop, :setup, :off]

  module MachineStates
    ON = 0
    NOTREADY = 1
    READY = 2
    WAITINGFORJOBS = 3
    PROCESSING = 4
    MAINTENANCE = 5
    ERROR = 6
    HALT = 7
    ESTOP = 8
    SETUP = 9
    OFF = 10
  end

  JOB_STATES = {started: 3, aborted: 4, finished: 5, postponed: 6, skipped: 7, restarted: 8}
  JOB_STATES_INVERTED = JOB_STATES.invert

  PRODUCTIVITY_COLORS = {poor: "#CC0000", ok: "#FB8C00", good: "#68CF02", great: "#008000", off: "#808080", throughput: "#1A5DAD"}

  ROUTING_KICKOUT_REASONS = {
    noValidPath: {database_key: 'no_valid_path'},
    routedWhileWaitingOtherProcess: {database_key: 'routed_while_waiting_other_process'},
    failFetchProcesses: {database_key: 'fail_fetch_processes'},
    markedDead: {database_key: 'marked_dead'},
    noProcesses: {database_key: 'no_processes'},
    receivedInvalidProcess: {database_key: 'received_invalid_process'},
    noProcessesAtNonStart: {database_key: 'no_processes_at_non_start'},
    manualReject: {database_key: 'manual_reject'},
    processAborted: {database_key: 'process_aborted'},
    timeSyncError: {database_key: 'time_sync_error'},
    fetchJobInfFailed: {database_key: 'fetch_job_inf_failed'},
    failedToSynchronize: {database_key: 'failed_to_synchronize'},
    returnJobToTray: {database_key: 'rework_complete'},
    failToScan: {database_key: 'other'},
    receivedStartInReject: {database_key: 'other'},
    completedInvalidProcessInDefault: {database_key: 'other'},
    other: {database_key: 'other'}
  }

  RETRIEVE_KICKOUT_REASONS = {
    noValidPath: {database_key: 'no_valid_path'},
    routedWhileWaitingOtherProcess: {database_key: 'routed_while_waiting_other_process'},
    failFetchProcesses: {database_key: 'fail_fetch_processes'},
    markedDead: {database_key: 'marked_dead'},
    noProcesses: {database_key: 'no_processes'},
    receivedInvalidProcess: {database_key: 'received_invalid_process'},
    noProcessesAtNonStart: {database_key: 'no_processes_at_non_start'},
    manualReject: {database_key: 'manual_reject'},
    processAborted: {database_key: 'process_aborted'},
    timeSyncError: {database_key: 'time_sync_error'},
    fetchJobInfFailed: {database_key: 'fetch_job_inf_failed'},
    failedToSynchronize: {database_key: 'failed_to_synchronize'},
    returnJobToTray: {database_key: 'rework_complete'},
    other: {database_key: 'other'}
  }

  def routing_kickout_reason_database_keys
    h = {}
    ROUTING_KICKOUT_REASONS.each do |k,v|
      h[v[:database_key]] = k
    end
    h
  end

  def routing_kickout_database_keys_arr
    routing_kickout_reason_database_keys.stringify_keys.keys
  end

  def post_job_state(sn, job_number, side, state, process, wait: nil)
    begin
      job_state_params = {
                            machine_sn: sn,
                            time: "#{Time.now.to_date} #{Time.now.to_time}",
                            job: {
                              job_number: job_number,
                              side: side,
                            },
                            job_state: {
                              state: state,
                              process: process
                            }
                         }

      if wait
        MachineInterfaceJobs::JobStateInterface::AddJobStateJob.set(wait: wait).perform_later(job_state_params)
      else
        MachineInterfaceJobs::JobStateInterface::AddJobStateJob.perform_later(job_state_params)
      end
    rescue => e
      Rails.logger.error e
    end
  end

  def value_for_machine_state_group(states_in_group, machine_states)
    states_in_group.reduce({
                             count: 0,
                             percent: 0
                           }) do |acc, state|
      if machine_states[state].nil? || machine_states[state][:percent].nil? || machine_states[state][:count].nil?
        acc
      else
        acc.merge!(machine_states[state]) { |k, a, b| a + b}
      end
    end
  end
end
