class JobProcessParametersController < ApplicationController
  include MachineInterfaceable
  include MachineDataViewFilter
  include RxModule::Tableable

  wrap_parameters :job_process_parameter, include: [:job, :lensside, :process, :batchId, :params]

  model_data_store(JobProcessParameter,
                   {create: false, read: true, update: false, delete: false, index: true},
                   {create: false, read: true, update: false, delete: false, index: true})

  def before_set_targets
    models = if filters["sn"]
      JobProcessParameter.joins(:job_process => :machine).where(job_processes: {machines: {sn: filters["sn"]}})
    else
      JobProcessParameter.joins(:job_process)
    end
    view_before_set_targets(models, params, "job_processes")
  end

  def before_filter(filter)
    view_before_filter(filter)
  end

  def add
    begin
      MachineInterfaceJobs::JobProcessInterface::AddJobProcessJob.perform_later(add_params)
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  private

  def add_params
    params[:job_process_parameter] = params[:processparameter] if params[:processparameter]

    job_process = params.require(:job_process_parameter).permit(:job, :lensside, :batchId, :process)
    process_data = params.require(:job_process_parameter).permit(params: [param: [:name, :value]])[:params][:param]

    add_params = {
      job_process: {
        manufacturing_process: job_process[:process]
      },
      process_data: process_data
    }

    add_params.merge! query_params
    add_params.merge! job_params(job_process)

    add_params
  end
end
