class JobStatesController < ApplicationController
  include MachineInterfaceable
  include MachineDataViewFilter
  include RxModule::Tableable
  include DataHelpers

  def self.set_target_before
    super + [:process_params]
  end

  wrap_parameters :job_state, include: [:job, :job_id, :machine_id, :batchId, :lensside, :substate, :state, :process, :comment, :scrap, :scrapReason]
  model_data_store(JobState,
                   {create: false, read: true, update: false, delete: false, index: true},
                   {create: false, read: true, update: false, delete: false, index: true})

  def before_set_targets
    machine_sn = filters["sn"]
    models = if machine_sn
               @models.where(machine: Machine.find_by(sn: machine_sn))
             else
               @models
             end
    models.includes(machine: :machine_groups, job: [])
    models = models.eager_load(:job)
    view_before_set_targets(models, params)
  end

  def before_filter(filter)
    if filter["id"] == "job_number"
      filter["id"] = "jobs.number"
    end
    view_before_filter(filter)
  end

  def before_sort(sort)
    if sort["id"] == "job_number"
      sort["id"] = "jobs.number"
    end
    if sort["id"] == "side"
      sort["id"] = "job_states.side"
    end
    sort
  end

  def add
    begin
      MachineInterfaceJobs::JobStateInterface::AddJobStateJob.perform_later(add_params)
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  def add_bulk
    begin
      AddBulkJobState.perform_later(job_state_add_bulk_params(params))
    rescue => e
      Rails.logger.error e
    end
    send_machine_response
  end

  def process_params
    render :process_params
  end

  private

  def add_params
    params[:job_state] = params[:jobstate] if params[:jobstate]
    job_state = if params.is_a?(Hash)
                  params[:job_state]
                else
                  params.require(:job_state).permit(:job, :batchId, :lensside, :state, :process, :substate, :comment, :scrap, :scrapReason, :abortCode)
                end

    add_params = {
      job_state: {
        state: job_state[:state],
        process: job_state[:process],
        substate: job_state[:substate],
        comment: job_state[:comment],
        scrap: job_state[:scrap],
        scrap_reason: job_state[:scrapReason],
        abort_code: job_state[:abortCode]
      }
    }

    add_params.merge! query_params
    add_params.merge! job_params(job_state)

    add_params
  end

  def job_state_add_bulk_params(params)
    {
      should_broadcast: params[:should_broadcast],
      job_states: params[:jobStates].map do |payload|
        searchParams = payload[:searchParams]
        job_state_data = payload[:jobState]
        side_num = Sideable.side_letter_to_side_num(job_state_data[:lensside])
        {
          machine: Machine.find_or_create_by(sn: searchParams[:sn]),
          time: "#{searchParams[:date]} #{searchParams[:time]}",
          job: FindHelpers::JobFinders.find_active_or_create_by_number_and_side_num(job_state_data[:job], side_num),
          side: side_num,
          state: job_state_data[:state].downcase,
          comment: job_state_data[:comment],
          scrap_reason: job_state_data[:scrapReason],
          substate: job_state_data[:substate],
          process: job_state_data[:process],
          abort_code: job_state_data[:abortCode]
        }
      end
    }
  end
end
