class TemplateReportController < ApplicationController
  include ReportMachines

  def single_report_generate
    report_params = template_report_params

    data = resolve_report_params(report_params)
    return render json: {error: "Invalid Time Selection"}, status: :unprocessable_entity if
      data[:timeframe] == :daily &&
        !valid_daily_time_range?(data[:report_start], data[:report_end])

    GenerateTemplateReportJob.perform_later(report_params)
    render json: data, status: :created
  rescue => e
    Rails.logger.error e
    render status: :internal_server_error
  end

  def generate_template_reports
    template_reports = template_reports_params
    data = template_reports.map do |report_params|
      generate(report_params)
    end
    render json: data, status: :created
  rescue => e
    Rails.logger.error e
    render status: :internal_server_error
  end

  def delete_template_report_jobs
    params = delete_template_report_jobs_params
    DelayedJobsHelpers.kill_delayed_jobs(params[:delayed_job_ids])
    render json: { success: true }, status: :ok
  rescue => e
    Rails.logger.error e
    render json: { success: false }, status: :unprocessable_entity
  end

  private

  def generate(report_params)
    data = resolve_report_params(report_params)
    return render json: {error: "Invalid Time Selection"}, status: :unprocessable_entity if
      data[:timeframe] == :daily &&
        !valid_daily_time_range?(data[:report_start], data[:report_end])

    delayed_job_info = GenerateTemplateReportJob.perform_later(report_params)
    data.merge({ delayed_job_id: delayed_job_info.blank? ? nil : delayed_job_info.provider_job_id })
  end

  def valid_daily_time_range?(start_time, end_time)
    return false if start_time > end_time
    return false if end_time - start_time > 1.days.seconds
    true
  end

  def delete_template_report_jobs_params
    params.permit(:delayed_job_ids => [])
  end

  def template_reports_params
    params.permit!.require(:template_reports)
  end

  def template_report_params
    params.permit!.require(:template_report)
  end

  def resolve_report_params(report)
    report_params = JSON.parse(report[:params],{ symbolize_names: true })
    period = ReportPeriod.parse_report_range(report_params[:report_range])

    report_targets = resolve_machine_targets(report_params[:target_ids])
    report_targets = report_targets.join(",") if report_targets.is_a? Array

    {
      report_targets: report_targets,
      template_report_id: report[:id],
      timeframe: KpiHelpers::Timeframe::TIMEFRAME_TO_ENUM.key(period[:timeframe]),
      report_start: period[:report_start],
      report_end: period[:report_end],
    }
  end
end

