import * as React from "react";
import {observer} from "mobx-react-lite";
import {Box} from "@material-ui/core";
import {t} from "@lingui/macro";
import {MachineTypeModel} from "../../stores/MachineTypeStore";
import {ProductionSettingModel, ProductionSettingStore} from "../../stores/ProductionSettingStore";
import {InlineWarning} from "../generic/InlineWarning";
import {MachineManualDownload} from "../machine/machine_worsktation/MachineManualDownload";


type MachineManualsViewProps = {
    type: MachineTypeModel
}

export const MachineManualsView: React.FunctionComponent<MachineManualsViewProps> = observer(({type}) => {
    const settings = ProductionSettingStore.selectedModel as ProductionSettingModel;

    if(!settings.machine_manual_path) {
        return <InlineWarning text={t`Machine manuals have not yet been registered to this MES-360 installation. Please contact support.`}/>
    }

    if(!type || !type.has_machine_manuals){
        return <InlineWarning text={t`There are no manuals available for this machine type.`}/>
    }

    return (
        <Box>
            {type.has_operating_manuals &&
                <MachineManualDownload
                    label={t`Operating Manual`}
                    model={type}
                    manualType={"operating"}
                />
            }
            {type.has_maintenance_manuals &&
                <MachineManualDownload
                    label={t`Maintenance Manual`}
                    model={type}
                    manualType={"maintenance"}
                />
            }
        </Box>
    )
})