import * as React from "react";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {MachineDataViewTab} from "../MachineDataViewTab";
import {EventModel, EventStore} from "../../../stores/EventStore";
import {makeColumn} from "@rx-suite/ui";
import {MachineDataViewPanelProps} from "../MachineDataView";
import {ArrowDropDown, ArrowRight} from "@material-ui/icons";
import {AsyncEventData} from "../sub_components/AsyncEventData";
import {useServerTimeColumn} from "../../../hooks/table_columns/useServerTimeColumn";


export const EventsPanel: React.FunctionComponent<MachineDataViewPanelProps> = observer((props) => {
    const {machine} = props;

    const columns = [
        {
            Header: t`Machine Events`,
            columns: [
                {
                    expander: true,
                    Header: () => null,
                    Expander: ({ isExpanded, ...rest }) => {
                        if(rest.original) {
                            return(
                                <div>
                                    {isExpanded ? <ArrowDropDown/> : <ArrowRight/>}
                                </div>
                            )
                        } else {
                            return null
                        }
                    },
                },
                makeColumn('message', t`Message`, "string"),
                makeColumn('event_type', t`Type`, "string"),
                useServerTimeColumn()
            ]
        }];

    const eventDataSubComponent = (original: EventModel) => {
        return <AsyncEventData model={original} />;
    }

    return (
        <MachineDataViewTab
            columns={columns}
            machine={machine}
            enabledKey={'event'}
            tabName={'Event'}
            store={EventStore}
            subComponent={eventDataSubComponent}
            exportedFileName={'Events.csv'}
            exportEndpoint={'events'}
        />
    );
});
