import { t } from "@lingui/macro"
import {SystemSettingModel, SystemSettingStore, TimeModel, TimeStore} from "@rx-suite/stores";
import {Model} from "@rx-suite/data";
import moment, {Moment} from "moment/moment";


export const useServerTimeColumn = (header=t`Date - Time`, id="time") => {
    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;

    return (
        {
            Header: header,
            id: id,
            accessor: (model: Model) => {
                const digObj = (obj, keys) => keys.reduce((curr, key) => curr?.[key] ?? null, obj);
                const modelTime = digObj(model, id.split('.')) as Date | string;
                return systemSettingModel.printDateTime(convertToServerTime(modelTime));
            }
        }
    )
}

export function convertToServerTime(time: Date | string): Moment {

    const timeModel = TimeStore.selectedModel as TimeModel;
    if (!timeModel) {
        return moment(time);
    }

    const serverOffset = timeModel.serverUTCOffset;

    const timeMoment = moment(time).utc(false);

    return timeMoment.add(serverOffset, 'minutes');
}