import * as React from 'react';
import {observer} from "mobx-react-lite";
import {Box, createStyles, Divider, LinearProgress, Theme, Typography} from "@material-ui/core";
import {CheckboxField, Conditional, ModelCheckbox, ModelForm, ModelInput} from "@rx-suite/ui";
import {JobSettingStore} from "../../stores/JobSettingsStore";
import {t} from "@lingui/macro";
import {makeStyles} from "@material-ui/core/styles";

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        bold: {
            fontWeight: 'bold'
        },
        sectionTitle: {
            fontSize: '1.35rem'
        }
    })
);


export const JobSettingForm = observer(() => {
    const classes = useStyles();
    const settings = JobSettingStore.selectedModel;
    if(!settings) {
        return <LinearProgress />
    }

    const [truncateJobNumberEnabled, setTruncateJobNumberEnabled] = React.useState(settings.max_job_number_length != null);
    const handleToggleTruncateJobNumber = React.useCallback((e) => {
        if(!e.target.checked){
            settings.viewModel.max_job_number_length = null;
        }else{
            settings.viewModel.max_job_number_length = 7;
        }
        setTruncateJobNumberEnabled(e.target.checked)
    }, [settings]);


    return(
        <ModelForm model={settings} suppressMakeCopy suppressImportExport>
            <Typography className={classes.sectionTitle}>{t`Data from Interfaces`}</Typography>
            <Divider/>
            <Typography variant={'caption'}>{t`These settings only apply to new incoming data.`}</Typography>
            <Box>
                <CheckboxField
                    label={t`Truncate Incoming Job Number`}
                    checked={truncateJobNumberEnabled}
                    onChange={handleToggleTruncateJobNumber}
                    hint={
                        {
                            title: t`Truncate Incoming Job Number`,
                            children: <Box>
                                <Typography>{t`Job Numbers from machine interfaces will be truncated to the specified number of digits.`}</Typography>
                                <Box mt={1}>
                                    <Typography>{t`For example, with a limit of 5 digits, the number 153644 will be interpreted as 15364`}</Typography>
                                </Box>
                            </Box>
                        }
                    }
                />
            </Box>
            <Conditional condition={truncateJobNumberEnabled}>
                <Box ml={2} mt={-1}>
                    <ModelInput name={'max_job_number_length'}/>
                </Box>
            </Conditional>
            <Box>
                <ModelCheckbox
                    name={'auto_pair_job'}
                    hint={
                        {
                            title: t`Automatically Create Order for Matching Jobs`,
                            children: <Box>
                                <Typography>{t`When enabled, two job numbers with different sides will automatically be linked to the same order.`}</Typography>
                                <Box mt={1}>
                                    <Typography className={classes.bold}>{t`Warning: This setting should NOT be enabled if an LMS system is already sending order information.`}</Typography>
                                </Box>

                            </Box>

                        }
                    }
                />
            </Box>
        </ModelForm>
    )
});
