import {attribute, DataStore, model, Model} from "@rx-suite/data";
import {t} from "@lingui/macro";
import * as yup from "yup";
import {MachineTypeModel} from "./MachineTypeStore";

class ConsumableRecordModelDataStore<T extends Model> extends DataStore<T> {
    optionsForMachineType(type: MachineTypeModel) {
        switch(type.name){
            case "multi_flex_2":
            case "multi_flex_2e":
                return [
                    {label: t`Polishing Tool (Cell 1)`, value: 'POLISHING_TOOL_CELL_1' },
                    {label: t`Pre-polishing Tool (Cell 1)`, value: 'PRE_POLISHING_TOOL_CELL_1'},
                    {label: t`Polishing Tool (Cell 2)`, value: 'POLISHING_TOOL_CELL_2'},
                    {label: t`Pre-polishing Tool (Cell 2)`, value: 'PRE_POLISHING_TOOL_CELL_2'},
                    {label: t`Polishing Tool (Cell 3)`, value: 'POLISHING_TOOL_CELL_3'},
                    {label: t`Pre-polishing Tool (Cell 3)` , value: 'PRE_POLISHING_TOOL_CELL_3'}
                ]
            case "vft_orbit_2":
            case "vft_orbit":
                return [
                    {label: t`Fast Tool 1`, value: 'FT1' },
                    {label: t`Fast Tool 2`, value: 'FT2' },
                    {label: t`Milling Tool`, value: 'MT' },
                    {label: t`Cribbing Tool`, value: 'CT' },
                    {label: t`Engraving Tool`, value: 'ET' }
                ]
            default:
                return []
        }
    }
}


@model({
    storeClass: ConsumableRecordModelDataStore,
    transportLayer: "/api/consumable_records",
    permissionTarget: "ConsumableRecord",
    routingInfo: {}
})
export class ConsumableRecordModel extends Model {
    @attribute({label: t`Tool`})
    name = ""

    @attribute({label: t`Use Count`})
    current_use_count = 0

    @attribute({label: t`Max Use`})
    max_use_count = 0

    @attribute({label: t`First Use`})
    first_use_date = Date()

    @attribute({label: t`Last Use`})
    last_use_date = Date()

    @attribute({label: t`Retired`})
    retired = null

    @attribute({label: t`Details`})
    details = null

    @attribute({label: t`Machine ID`})
    machine_id = null

    get resolved_name(): string {
        const label: {[key: string]: string} = {
            'POLISHING_TOOL_CELL_1': t`Polishing Tool (Cell 1)`,
            'POLISHING_TOOL_CELL_2': t`Polishing Tool (Cell 2)`,
            'POLISHING_TOOL_CELL_3': t`Polishing Tool (Cell 3)`,
            'PRE_POLISHING_TOOL_CELL_1': t`Pre-polishing Tool (Cell 1)`,
            'PRE_POLISHING_TOOL_CELL_2': t`Pre-polishing Tool (Cell 2)`,
            'PRE_POLISHING_TOOL_CELL_3': t`Pre-polishing Tool (Cell 3)`,
            'FT1': t`Fast Tool 1`,
            'FT2': t`Fast Tool 2`,
            'MT': t`Milling Tool`,
            'CT': t`Cribbing Tool`,
            'ET': t`Engraving Tool`
        };

        if (!label[this.name]) {
            return t`Unknown Tool`;
        }

        return label[this.name];
    }

    constructor() {
        super();
        super.setup();
    }
}

export const ConsumableRecordStore = ConsumableRecordModel.store as unknown as DataStore<ConsumableRecordModelDataStore>