import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro";

import * as yup from 'yup';

@model({
    singleton: true,
    transportLayer: "/api/job_settings",
    permissionTarget: 'JobSetting',
    routingInfo: {
        update: "/settings/job_settings/:id/edit",
        read: "/settings/job_settings/:id"
    }
})
export class JobSettingModel extends Model {
    @attribute({
        label: t`Max Job Number Length`,
        validation: yup
            .number()
            .positive()
            .integer()
            .nullable()
            .transform((_, val) => val === Number(val) ? val : null)
    })
    max_job_number_length: number | null  = null;

    @attribute({
        label: t`Automatically Create Orders for Matching Jobs`,
        validation: yup.boolean()
    })
    auto_pair_job: boolean = false

    constructor() {
        super();
        super.setup();
    }
}

export const JobSettingStore = JobSettingModel.store as DataStore<JobSettingModel>;

