module MachineInterfaceJobs
  module Base
    class MachineInterfaceJob < ApplicationJob

      # Base Class for machine interface jobs that manipulate or create machine data

      class InvalidMachineInterfaceData < StandardError; end

      rescue_from(InvalidMachineInterfaceData) do |exception|
        # Do not retry these jobs as there is missing data and will not be able to complete regardless
        Rails.logger.error "Failed running job #{self.class.name} - missing or invalid data."
      end

      def perform(add_params)
        @machine = Machine.find_or_create_by(sn: add_params[:machine_sn])

        @model = build_model(add_params)

        if @model.save
          @model.broadcast_machine_data(@machine, async: false) # broadcast to other Rx-Modules
          @model.forward # forward data to other external systems

          after_model_save
        end
      end

      def after_model_save
        # Overwrite this method to allow the interface to generate KPIs
      end

      def build_model(add_params)
        raise NotImplementedError
      end
    end
  end
end