module MachineInterfaceJobs
  module JobStateInterface
    class AddJobStateJob < MachineInterfaceJobs::Base::JobDataJob
      include DataHelpers
      queue_as :high_priority

      # Resolver for /connector/satisloh/jobstate/1.0/add

      def build_model(add_params)
        JobState.new({
                       job: @job,
                       machine: @machine,
                       batch_id: add_params[:job][:batch_id],
                       process: add_params[:job_state][:process],
                       substate: add_params[:job_state][:substate],
                       side: @job.side_num,
                       state: resolve_job_state(add_params[:job_state][:state]),
                       comment: add_params[:job_state][:comment],
                       scrap: add_params[:job_state][:scrap],
                       scrap_reason: add_params[:job_state][:scrap_reason],
                       time: add_params[:time],
                       abort_code: add_params[:job_state][:abort_code]
                     })
      end

      def after_model_save
        KpiJobStateJob.perform_later(@model)
        AddBoxCoaterBatchFromJobStateJob.perform_later(@model) if @model.is_box_coater_event?
      end

      def resolve_job_state(state)
        raise InvalidMachineInterfaceData if state.blank?

        symbol = if state.is_a? String
                   state.downcase.to_sym
                 elsif state.is_a? Symbol
                   state
                 else
                   raise InvalidMachineInterfaceData
                 end

        JOB_STATES[symbol]
      end
    end
  end
end


