module MachineKpiCalculator
  class EnergyUsageBreakdown < MachineKpiCalculator

    include KpiTableHandler

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options)
        kpi = EnergyUsageBreakdown.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :machine_states,
          handler: KpiTableHandler::MachineStateDurationKpiHandler,
          params: {
            machine_states: %w[on notready ready waitingforjobs processing maintenance error halt estop setup off]
          }
        }
      ]
    end

    def build_results
      get_energy_per_state
    end

    def build_results_per_machine
      get_energy_per_state
    end

    def get_energy_per_state
      data[:machine_states].each do |machine_id, states|
        machine_type = Machine.find_by_id(machine_id).machine_type

        states.each do |state_sym, value|
          if timeframe == Timeframe::ALL
            if machine_type.nil?
              states[state_sym] = 0
              next
            end

            energy_consumption = get_state_consumption(state_sym, machine_type, value)

            states[state_sym] = energy_consumption
          else
            value.each do |state, count|
              if machine_type.nil?
                states[state_sym][state] = 0
                next
              end

              energy_consumption = get_state_consumption(state, machine_type, count)

              states[state_sym][state] = energy_consumption
            end

          end
        end

        states
      end
    end

    def get_state_consumption(state, machine_type, state_duration)
      machine_state = state.to_s
      energy_per_hour = machine_type.has_energy_consumptions? ? machine_type.energy_consumptions[machine_state.to_sym] : 0
      state_time_in_hours = state_duration ? state_duration / 3600 : 0
      energy_consumption = energy_per_hour ? (energy_per_hour * state_time_in_hours) : 0

      energy_consumption
    end

  end
end
