module MachineKpiCalculator
  class ErrorTrends < TopErrors

    class << self
      def get_for_period_and_machines(start_time, end_time, timeframe, machine_ids, options)
        kpi = ErrorTrends.new(start_time, end_time, timeframe, machine_ids, options)
        kpi.run
      end
    end

    def initialize(start_time, end_time, timeframe, machine_ids, options)
      super(start_time, end_time, timeframe, machine_ids, options)
    end

    def handlers
      [
        {
          key: :error_count,
          handler: KpiTableHandler::MachineErrorCountKpiHandler,
          params: {
          }
        }
      ]
    end

    def build_results_per_machine
      resolved_error_results = {}
      data[:error_count].each do |machine_id, error_count|
        resolved_error_results[machine_id] = {}

        error_codes = group_codes(error_count)

        # get top errors from error codes
        top_errors = extract_top_x(error_codes, 5)
        top_errors = top_errors.map {|e| e[:code]} # We only need the code of the top 5 errors

        results = {}
        top_errors.each do |error|
          results[error] = {}
        end

        error_count.each do |time, err_data|
          results.keys.each { |err_code| results[err_code][time] =  0}
          err_data.each do |err_code, val|
            next unless top_errors.include?(err_code)
            results[err_code][time] = val
          end
        end

        results.each do |code, time_hash|
          resolved_error_results[machine_id][resolve_error_name(code, machine_id)] ||= {}
          resolved_error_results[machine_id][resolve_error_name(code, machine_id)].merge!(time_hash) {|k, val1, val2| val1 + val2}
        end
      end
      resolved_error_results
    end
  end
end