# == Schema Information
#
# Table name: machine_processes
#
#  id             :bigint           not null, primary key
#  code           :string(4000)
#  display_name   :string(4000)
#  rank           :integer
#  system_default :boolean
#  created_at     :datetime         not null
#  updated_at     :datetime         not null
#
class MachineProcess < ApplicationRecord
  has_many :machines
  has_many :machine_types
  validates :code, :rank, uniqueness: true, presence: true

  class << self
    def seed
      puts "Seeding Machine Processes..."

      MachineProcess.create_with(display_name: "Block Picking", rank: 10, system_default: true).find_or_create_by(code: 'block_picking')
      MachineProcess.create_with(display_name: "Blocking", rank: 20, system_default: true).find_or_create_by(code: 'blocking')
      MachineProcess.create_with(display_name: "Generating", rank: 30, system_default: true).find_or_create_by(code: 'generating')
      MachineProcess.create_with(display_name: "Polishing", rank: 40, system_default: true).find_or_create_by(code: 'polishing')
      MachineProcess.create_with(display_name: "Engraving", rank: 50, system_default: true).find_or_create_by(code: 'engraving')
      MachineProcess.create_with(display_name: "Coating (Spin)", rank: 60, system_default: true).find_or_create_by(code: 'coating_spin')
      MachineProcess.create_with(display_name: "Deblocking", rank: 70, system_default: true).find_or_create_by(code: 'deblocking')
      MachineProcess.create_with(display_name: "Coating (Dip)", rank: 80, system_default: true).find_or_create_by(code: 'coating_dip')
      MachineProcess.create_with(display_name: "Coating (AR)", rank: 90, system_default: true).find_or_create_by(code: 'coating_ar')
      MachineProcess.create_with(display_name: "Edging", rank: 100, system_default: true).find_or_create_by(code: 'edging')
    end
  end
end
