# == Schema Information
#
# Table name: machine_types
#
#  id                 :bigint           not null, primary key
#  category           :string(4000)
#  display_name       :string(4000)
#  name               :string(4000)     not null
#  seed_version       :integer          default(0)
#  system_default     :boolean          default(FALSE)
#  created_at         :datetime         not null
#  updated_at         :datetime         not null
#  machine_process_id :bigint
#
# Indexes
#
#  index_machine_types_on_machine_process_id  (machine_process_id)
#
# Foreign Keys
#
#  fk_rails_...  (machine_process_id => machine_processes.id)
#
class MachineType < ApplicationRecord
  has_many :machines
  has_many :error_filters, as: :filter_target
  has_many :abort_filters, as: :filter_target
  belongs_to :machine_process, required: false

  concerning :MachineTypeData do
    # Data derived entirely from the machine type class

    def type_data_handler
      case name
        # Block Pickers
      when 'art_block_pod'
        return MachineTypes::Others::ArtBlockPod

        # Blockers
      when 'art_blocker_a'
        return MachineTypes::Blockers::ArtBlockerA
      when 'art_blocker_m'
        return MachineTypes::Blockers::ArtBlockerM
      when 'layoutblocker_pra'
        return MachineTypes::Blockers::LayoutblockerPra

        # Box Coaters
      when '1200_dlx_2'
        return MachineTypes::BoxCoaters::Dlx21200
      when 'mc_280_x'
        return MachineTypes::BoxCoaters::Mc280X
      when 'mc_380_x_2'
        return MachineTypes::BoxCoaters::Mc380X2
      when '1500_x'
        return MachineTypes::BoxCoaters::X1500

        # Deblockers
      when 'art_deblocker_a'
        return MachineTypes::Deblockers::ArtDeblockerA
      when 'art_deblocker_a_2'
        return MachineTypes::Deblockers::ArtDeblockerA2
      when 'art_deblocker_m'
        return MachineTypes::Deblockers::ArtDeblockerM

        # Edgers
      when 'mei_edger'
        return MachineTypes::Edgers::MeiEdger

        # Engravers
      when 'lens_engraver_uv'
        return MachineTypes::Engravers::LensEngraverUv

        # Generators
      when 'vft_macro'
        return MachineTypes::Generators::VftMacro
      when 'vft_macro_e'
        return MachineTypes::Generators::VftMacroE
      when 'vft_orbit'
        return MachineTypes::Generators::VftOrbit
      when 'vft_orbit_2'
        return MachineTypes::Generators::VftOrbit2
      when 'vft_orbit_2e'
        return MachineTypes::Generators::VftOrbit2e

        # Polishers
      when 'duo_flex'
        return MachineTypes::Polishers::DuoFlex
      when 'multi_flex'
        return MachineTypes::Polishers::MultiFlex
      when 'multi_flex_2'
        return MachineTypes::Polishers::MultiFlex2
      when 'multi_flex_2e'
        return MachineTypes::Polishers::MultiFlex2e
      when 'toro_flex'
        return MachineTypes::Polishers::ToroFlex

        # Others

      when 'art_block_pod'
        return MachineTypes::Others::ArtBlockPod
      else
        return MachineTypes::Others::Generic
      end
    end

    def image_url
      type_data_handler.image_url
    end

    def translator_class
      type_data_handler.translator_class
    end

    def interfaces
     type_data_handler.interfaces
    end

    def has_consumable_data?
      type_data_handler.has_consumable_data?
    end

    def consumable_data_params
      type_data_handler.consumable_data_params
    end

    def has_machine_manuals?
      type_data_handler.has_machine_manuals?
    end

    def machine_manuals
      type_data_handler.machine_manuals
    end

    def has_energy_consumptions?
      type_data_handler.has_energy_consumptions?
    end

    def energy_consumptions
      type_data_handler.energy_consumptions
    end
  end

  def should_broadcast?
    true
  end
end
