module ReportComponents
  class PotentialEnergySavings < ReportComponent
    include ReportComponentHelpers
    include KpiHelpers
    include DataHelpers

    def component_id
      'potential_energy_savings'
    end

    def use_timeline?
      timeframe == Timeframe::HOUR_BY_HOUR
    end

    def component_type
      if use_timeline?
        'line'
      else
        'bar'
      end
    end

    def generate(args)
      if use_timeline?
        energy_savings_timeline
      else
        energy_savings_count
      end
    end

    def energy_savings_timeline
      idle_states =  MachineKpi.idle_time_runs(start_time,
                                                end_time,
                                                machine_ids,
                                                options: {aggregate_machines: false})
      unproductive_instances = 0
      total_potential_savings = 0
      data = []

      idle_states.each do |machine_id, idle_periods|
        machine = Machine.find_by(id: machine_id)
        machine_name = machine.ensured_name

        if idle_periods.empty?
          data = insert_holder_in_data(data, machine.ensured_name)
        end

        idle_periods.map do |idle|
          idle_duration = (idle[:duration][:count] / 60).to_f
          idle_energy = idle[:energy]

          # Append idle data if idle_duration is greater than 30 mins
          unproductive_instances += 1
          total_potential_savings += idle_energy
          data << {
            name: "#{machine_name} - Idle",
            x: machine_name,
            y: [idle[:start], idle[:end]],
            duration: idle_duration.round(0),
            fillColor: PRODUCTIVITY_COLORS[:ok]
          }
        end
      end

      {
        options: {
          graphVariant: 'multi-machine-timeline'
        },
        unproductive_instances: unproductive_instances,
        total_potential_savings: total_potential_savings.round(2),
        series: [
          {
            name: "Potential Savings",
            data: data
          }
        ]
      }
    end

    def energy_savings_count
      idle_states =  MachineKpi.idle_time_counts(start_time,
                                                 end_time,
                                                 timeframe,
                                                 machine_ids,
                                                 options: {aggregate_machines: false})

      unproductive_instances = 0
      total_potential_savings = 0
      categories = []
      series = []

      idle_states.each do |period, values|
        categories << bucket_time_to_timeframe_label(period)
        values.each do |machine_id, machine_data|
          machine = Machine.find_by(id: machine_id)
          machine_name = machine.ensured_name
          if series.find {|serie| serie[:name] == machine_name}.nil?
            series << {name: machine_name, data: []}
          end

          series.find {|serie| serie[:name] == machine_name}[:data] << machine_data[:count][:count]

          unproductive_instances += machine_data[:count][:count]
          total_potential_savings += machine_data[:energy][:count]
        end
      end

      {
        options: {
          graphVariant: 'multi-machine-count'
        },
        unproductive_instances: unproductive_instances,
        total_potential_savings: total_potential_savings.round(2),
        categories: categories,
        series: series,
        fillColor: PRODUCTIVITY_COLORS[:ok]
      }
    end

    def insert_holder_in_data(data, machine_name)
      data << {name: "#{machine_name} - Holder", x: machine_name, y: [@start_time, @start_time]}
    end

  end
end